/*
 * This file is part of TechReborn, licensed under the MIT License (MIT).
 *
 * Copyright (c) 2018 TechReborn
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

package techreborn.blockentity.machine.tier1;

import org.apache.commons.lang3.tuple.Pair;
import reborncore.api.IToolDrop;
import reborncore.api.blockentity.InventoryProvider;
import reborncore.client.containerBuilder.IContainerProvider;
import reborncore.client.containerBuilder.builder.BuiltContainer;
import reborncore.client.containerBuilder.builder.ContainerBuilder;
import reborncore.common.powerSystem.PowerAcceptorBlockEntity;
import reborncore.common.registration.RebornRegister;
import reborncore.common.registration.config.ConfigRegistry;
import reborncore.common.util.IInventoryAccess;
import reborncore.common.util.RebornInventory;
import reborncore.common.util.ItemUtils;
import techreborn.TechReborn;
import techreborn.init.ModSounds;
import techreborn.init.TRContent;
import techreborn.utils.RecipeUtils;
import techreborn.init.TRBlockEntities;

import javax.annotation.Nullable;
import net.minecraft.class_1657;
import net.minecraft.class_1703;
import net.minecraft.class_1715;
import net.minecraft.class_1799;
import net.minecraft.class_1856;
import net.minecraft.class_1860;
import net.minecraft.class_2350;
import net.minecraft.class_2371;
import net.minecraft.class_2487;
import net.minecraft.class_3419;
import net.minecraft.class_3956;
import java.util.ArrayList;
import java.util.List;

/**
 * Created by modmuss50 on 20/06/2017.
 */
@RebornRegister(TechReborn.MOD_ID)
public class AutoCraftingTableBlockEntity extends PowerAcceptorBlockEntity
		implements IToolDrop, InventoryProvider, IContainerProvider {

	@ConfigRegistry(config = "machines", category = "autocrafter", key = "AutoCrafterInput", comment = "AutoCrafting Table Max Input (Value in EU)")
	public static int maxInput = 32;
	@ConfigRegistry(config = "machines", category = "autocrafter", key = "AutoCrafterMaxEnergy", comment = "AutoCrafting Table Max Energy (Value in EU)")
	public static int maxEnergy = 10_000;

	public RebornInventory<AutoCraftingTableBlockEntity> inventory = new RebornInventory<>(11, "AutoCraftingTableBlockEntity", 64, this, getInventoryAccess());
	public int progress;
	public int maxProgress = 120;
	public int euTick = 10;

	class_1715 inventoryCrafting = null;
	class_1860 lastCustomRecipe = null;
	class_1860 lastRecipe = null;

	public boolean locked = true;

	public AutoCraftingTableBlockEntity() {
		super(TRBlockEntities.AUTO_CRAFTING_TABLE);
	}

	@Nullable
	public class_1860 getIRecipe() {
		class_1715 crafting = getCraftingInventory();
		if (!crafting.method_5442()) {
			if (lastRecipe != null) {
				if (lastRecipe.method_8115(crafting, field_11863)) {
					return lastRecipe;
				}
			}
			for (class_1860 testRecipe : RecipeUtils.getRecipes(field_11863, class_3956.field_17545)) {
				if (testRecipe.method_8115(crafting, field_11863)) {
					lastRecipe = testRecipe;
					return testRecipe;
				}
			}
		}
		return null;
	}

	public class_1715 getCraftingInventory() {
		if (inventoryCrafting == null) {
			inventoryCrafting = new class_1715(new class_1703(null, -1) {
				@Override
				public boolean method_7597(class_1657 playerIn) {
					return false;
				}
			}, 3, 3);
		}
		for (int i = 0; i < 9; i++) {
			inventoryCrafting.method_5447(i, inventory.method_5438(i));
		}
		return inventoryCrafting;
	}

	public boolean canMake(class_1860<class_1715> recipe) {
		if (recipe != null && recipe.method_8113(3, 3)) {
			boolean missingOutput = false;
			int[] stacksInSlots = new int[9];
			for (int i = 0; i < 9; i++) {
				stacksInSlots[i] = inventory.method_5438(i).method_7947();
			}

			class_2371<class_1856> ingredients = recipe.method_8117();
			for (class_1856 ingredient : ingredients) {
				if (ingredient != class_1856.field_9017) {
					boolean foundIngredient = false;
					for (int i = 0; i < 9; i++) {
						class_1799 stack = inventory.method_5438(i);
						int requiredSize = locked ? 1 : 0;
						if (stack.method_7914() == 1) {
							requiredSize = 0;
						}
						if (stacksInSlots[i] > requiredSize) {
							if (ingredient.method_8093(stack)) {
								if (stack.method_7909().method_7858() != null) {
									if (!hasRoomForExtraItem(new class_1799(stack.method_7909().method_7858()))) {
										continue;
									}
								}
								foundIngredient = true;
								stacksInSlots[i]--;
								break;
							}
						}
					}
					if (!foundIngredient) {
						missingOutput = true;
					}
				}
			}
			if (!missingOutput) {
				if (hasOutputSpace(recipe.method_8110(), 9)) {
					return true;
				}
			}
			return false;
		}
		return false;
	}

	boolean hasRoomForExtraItem(class_1799 stack) {
		class_1799 extraOutputSlot = inventory.method_5438(10);
		if (extraOutputSlot.method_7960()) {
			return true;
		}
		return hasOutputSpace(stack, 10);
	}

	public boolean hasOutputSpace(class_1799 output, int slot) {
		class_1799 stack = inventory.method_5438(slot);
		if (stack.method_7960()) {
			return true;
		}
		if (ItemUtils.isItemEqual(stack, output, true, true)) {
			if (stack.method_7914() > stack.method_7947() + output.method_7947()) {
				return true;
			}
		}
		return false;
	}

	public boolean make(class_1860<class_1715> recipe) {
		if (recipe == null || !canMake(recipe)) {
			return false;
		}
		for (int i = 0; i < recipe.method_8117().size(); i++) {
			class_2371<class_1856> ingredients = recipe.method_8117();
			class_1856 ingredient = ingredients.get(i);
			// Looks for the best slot to take it from
			class_1799 bestSlot = inventory.method_5438(i);
			if (ingredient.method_8093(bestSlot)) {
				handleContainerItem(bestSlot);
				bestSlot.method_7934(1);
			} else {
				for (int j = 0; j < 9; j++) {
					class_1799 stack = inventory.method_5438(j);
					if (ingredient.method_8093(stack)) {
						handleContainerItem(stack);
						stack.method_7934(1); // TODO is this right? or do I need
											// to use it as an actull
											// crafting grid
						break;
					}
				}
			}
		}
		class_1799 output = inventory.method_5438(9);
		// TODO fire forge recipe event
		class_1799 ouputStack = recipe.method_8116(getCraftingInventory());
		if (output.method_7960()) {
			inventory.method_5447(9, ouputStack.method_7972());
		} else {
			// TODO use ouputStack in someway?
			output.method_7933(recipe.method_8110().method_7947());
		}
		return true;
	}

	private void handleContainerItem(class_1799 stack) {
		if (stack.method_7909().method_7857()) {
			class_1799 containerItem = new class_1799(stack.method_7909().method_7858());
			class_1799 extraOutputSlot = inventory.method_5438(10);
			if (hasOutputSpace(containerItem, 10)) {
				if (extraOutputSlot.method_7960()) {
					inventory.method_5447(10, containerItem.method_7972());
				} else if (ItemUtils.isItemEqual(extraOutputSlot, containerItem, true, true)
						&& extraOutputSlot.method_7914() < extraOutputSlot.method_7947() + containerItem.method_7947()) {
					extraOutputSlot.method_7933(1);
				}
			}
		}
	}

	public boolean hasIngredient(class_1856 ingredient) {
		for (int i = 0; i < 9; i++) {
			class_1799 stack = inventory.method_5438(i);
			if (ingredient.method_8093(stack)) {
				return true;
			}
		}
		return false;
	}

	public boolean isItemValidForRecipeSlot(class_1860 recipe, class_1799 stack, int slotID) {
		if (recipe == null) {
			return true;
		}
		int bestSlot = findBestSlotForStack(recipe, stack);
		if (bestSlot != -1) {
			return bestSlot == slotID;
		}
		return true;
	}

	public int findBestSlotForStack(class_1860 recipe, class_1799 stack) {
		if (recipe == null) {
			return -1;
		}
		class_2371<class_1856> ingredients = recipe.method_8117();
		List<Integer> possibleSlots = new ArrayList<>();
		for (int i = 0; i < recipe.method_8117().size(); i++) {
			class_1799 stackInSlot = inventory.method_5438(i);
			class_1856 ingredient = ingredients.get(i);
			if (ingredient != class_1856.field_9017 && ingredient.method_8093(stack)) {
				if (stackInSlot.method_7960()) {
					possibleSlots.add(i);
				} else if (stackInSlot.method_7909() == stack.method_7909()) {
					if (stackInSlot.method_7914() >= stackInSlot.method_7947() + stack.method_7947()) {
						possibleSlots.add(i);
					}
				}
			}
		}
		// Slot, count
		Pair<Integer, Integer> smallestCount = null;
		for (Integer slot : possibleSlots) {
			class_1799 slotStack = inventory.method_5438(slot);
			if (slotStack.method_7960()) {
				return slot;
			}
			if (smallestCount == null) {
				smallestCount = Pair.of(slot, slotStack.method_7947());
			} else if (smallestCount.getRight() >= slotStack.method_7947()) {
				smallestCount = Pair.of(slot, slotStack.method_7947());
			}
		}
		if (smallestCount != null) {
			return smallestCount.getLeft();
		}
		return -1;
	}

	public int getProgress() {
		return progress;
	}

	public void setProgress(int progress) {
		this.progress = progress;
	}

	public int getMaxProgress() {
		if (maxProgress == 0) {
			maxProgress = 1;
		}
		return maxProgress;
	}

	public void setMaxProgress(int maxProgress) {
		this.maxProgress = maxProgress;
	}

	// TilePowerAcceptor
	@Override
	public void method_16896() {
		super.method_16896();
		if (field_11863.field_9236) {
			return;
		}
		class_1860 recipe = getIRecipe();
		if (recipe != null) {
			if (progress >= maxProgress) {
				if (make(recipe)) {
					progress = 0;
				}
			} else {
				if (canMake(recipe)) {
					if (canUseEnergy(euTick)) {
						progress++;
						if (progress == 1) {
							field_11863.method_8465(null, field_11867.method_10263(), field_11867.method_10264(), field_11867.method_10260(), ModSounds.AUTO_CRAFTING,
									class_3419.field_15245, 0.3F, 0.8F);
						}
						useEnergy(euTick);
					}
				} else {
					progress = 0;
				}
			}
		}
		if (recipe == null) {
			progress = 0;
		}
	}

	// Easyest way to sync back to the client
	public int getLockedInt() {
		return locked ? 1 : 0;
	}

	public void setLockedInt(int lockedInt) {
		locked = lockedInt == 1;
	}

	@Override
	public double getBaseMaxPower() {
		return maxEnergy;
	}

	@Override
	public double getBaseMaxOutput() {
		return 0;
	}

	@Override
	public double getBaseMaxInput() {
		return maxInput;
	}

	@Override
	public boolean canAcceptEnergy(class_2350 enumFacing) {
		return true;
	}

	@Override
	public boolean canProvideEnergy(class_2350 enumFacing) {
		return false;
	}

	@Override
	public class_2487 method_11007(class_2487 tag) {
		tag.method_10556("locked", locked);
		return super.method_11007(tag);
	}

	@Override
	public void method_11014(class_2487 tag) {
		if (tag.method_10545("locked")) {
			locked = tag.method_10577("locked");
		}
		super.method_11014(tag);
	}

	// TileMachineBase
	@Override
	public boolean canBeUpgraded() {
		return false;
	}

	private static IInventoryAccess<AutoCraftingTableBlockEntity> getInventoryAccess(){
		return (slotID, stack, facing, direction, blockEntity) -> {
			switch (direction){
				case INSERT:
					if (slotID > 8) {
						return false;
					}
					int bestSlot = blockEntity.findBestSlotForStack(blockEntity.getIRecipe(), stack);
					if (bestSlot != -1) {
						return slotID == bestSlot;
					}
					return true;
				case EXTRACT:
					return slotID > 8;
			}
			return true;
		};
	}

	// This machine doesnt have a facing
	@Override
	public class_2350 getFacingEnum() {
		return class_2350.field_11043;
	}

	// IToolDrop
	@Override
	public class_1799 getToolDrop(class_1657 playerIn) {
		return TRContent.Machine.AUTO_CRAFTING_TABLE.getStack();
	}

	// ItemHandlerProvider
	@Override
	public RebornInventory getInventory() {
		return inventory;
	}

	// IContainerProvider
	@Override
	public BuiltContainer createContainer(int syncID, class_1657 player) {
		return new ContainerBuilder("autocraftingtable").player(player.field_7514).inventory().hotbar().addInventory()
				.blockEntity(this).slot(0, 28, 25).slot(1, 46, 25).slot(2, 64, 25).slot(3, 28, 43).slot(4, 46, 43)
				.slot(5, 64, 43).slot(6, 28, 61).slot(7, 46, 61).slot(8, 64, 61).outputSlot(9, 145, 42)
				.outputSlot(10, 145, 70).syncEnergyValue().syncIntegerValue(this::getProgress, this::setProgress)
				.syncIntegerValue(this::getMaxProgress, this::setMaxProgress)
				.syncIntegerValue(this::getLockedInt, this::setLockedInt).addInventory().create(this, syncID);
	}

	@Override
	public boolean hasSlotConfig() {
		return false;
	}
}
